# Generated by Django 2.1 on 2018-08-03 03:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0002_auto_20180803_0515")]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="sites",
            field=models.ManyToManyField(blank=True, to="sites.Site"),
        )
    ]
