# Generated by Django 2.2.6 on 2019-10-21 22:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0007_auto_20181025_0150")]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="job_title",
            field=models.CharField(blank=True, max_length=100, null=True),
        )
    ]
