ACCOUNT_MANAGER_ROLE = "account_manager"
AUDITOR_ROLE = "auditor"
CLINICIAN_ROLE = "clinician"
CLINICIAN_SUPER_ROLE = "clinician_super"
CUSTOM_ROLE = "custom"
NURSE_ROLE = "research_nurse"
STAFF_ROLE = "staff"
STATISTICIAN = "statistician"
