# Generated by Django 3.2.13 on 2022-08-25 01:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_auth", "0026_rename_permissions_edcpermissions"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="edcpermissions",
            options={
                "verbose_name": "Edc Permissions",
                "verbose_name_plural": "Edc Permissions",
            },
        ),
    ]
