# Generated by Django 2.2.6 on 2019-10-29 17:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_auth", "0017_auto_20191026_2231")]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="roles",
            field=models.ManyToManyField(blank=True, to="edc_auth.Role"),
        )
    ]
